#include "libxml/globals.h"
#include "libxml/valid.h"
#include <vector>
#include <string>
#include <iostream>
using namespace std;

struct ExecuterData {
  string jobDefID;
  string jobName;
  string jobExeID;
};

struct jobData {
  string jobDefID;
  string jobName;
  string jobExeID;
  string infoExecuter;
};

struct cpuConsumption {
  string count;
  string unit;
};

struct diskConsumption {
  string count;
  string unit;
};

struct fileInfo {
  string LFN;
  string type;
  string logCol;
  vector<string> turlList;
  vector<string> SEList;
};

struct actualPar {
  string name;
  string value;
};

struct jobDef {
  string jobDefID;
  string transImpl;
  string transUses;
  string transVersion;
  string jobName;
  cpuConsumption CPU;
  diskConsumption disk;
  vector<fileInfo> jobInputs;
  vector<fileInfo> jobOutputs;
  vector<actualPar> jobPars;
};
  
int main( int argc, char* argv[] ) {
  
  if(argc != 2) {
    cout << "Too few or too many arguments" << endl;
    exit(1);
  }

  xmlInitParser();

  xmlDocPtr xmldoc = xmlParseFile( argv[1] );
  if( xmldoc == NULL ) {
    cout << "Could not parse " << argv[1] << endl;
    exit(1);
  }

  /*  xmlDtdPtr xmldtd;
  if( argv[2] != NULL ) {
    xmldtd = xmlParseDTD(NULL,(const xmlChar*)argv[2]);
    if( xmldtd == NULL ) {
      cout << "dtd not properly loaded" << endl;
      exit(1);
    }

    xmlValidCtxtPtr validtxt;
    int parsing = xmlValidateDtd( validtxt, xmldoc, xmldtd );
    if( parsing == 0 ) {
      cout << "Document " << xmldoc->URL << " not a valid document according to the dtd " << xmldtd->SystemID << endl;
      exit(1);
    }
    } */

  //  cout << "version: " << xmldoc->version << endl;
  //  cout << "element-type: " << xmldoc->type << endl;
  //  cout << "encoding: " << xmldoc->encoding << endl;

  xmlNodePtr xmlroot,xmlnode,xmliterator,xmliterator2,xmliterator3;
  xmlroot = xmlDocGetRootElement( xmldoc );
  // Get attributes for root-element
  xmlAttrPtr xmlattr;
  xmlattr = xmlroot->properties;

  // not necessary
  while( xmlattr != NULL ) {
    cout << "Name: " << (const char*)xmlattr->name << endl;
    cout << "Content: " << (const char*)xmlattr->xmlChildrenNode->content << endl;
    xmlattr = xmlattr->next;
  }
  
  xmlroot = xmlroot->xmlChildrenNode->next;

  // defining variables for the parsing
  vector<ExecuterData> vExecuterData;
  vector<jobData> vJobExeData;
  vector<jobDef> vJobDef;

  if( !strcmp( (const char*)xmlroot->name, "getExecuterData" ) ) {
    xmlnode = xmlroot->xmlChildrenNode->next;
    while( xmlnode != NULL ) {
      
      xmliterator = xmlnode->xmlChildrenNode->next;
      ExecuterData data;
      
      while( xmliterator != NULL ) {
	if( !strcmp( (char*)xmliterator->name,"jobDefID") ) 
	  if( xmliterator->xmlChildrenNode != NULL ) data.jobDefID = (char*)xmliterator->xmlChildrenNode->content;
	if( !strcmp( (char*)xmliterator->name,"jobName") ) 
	  if( xmliterator->xmlChildrenNode != NULL ) data.jobName = (char*)xmliterator->xmlChildrenNode->content;

	xmliterator = xmliterator->next->next;

      }

      vExecuterData.push_back( data );
      xmlnode = xmlnode->next->next;
    }

    // print what has been read
    for( unsigned int i=0; i<vExecuterData.size(); i++ ) {
      cout << "jobDefID[" << i << "] : " << vExecuterData[i].jobDefID << endl;
      cout << "jobName [" << i << "] : " << vExecuterData[i].jobName << endl;
    }
  }

  if( (!strcmp( (char*)xmlroot->name, "getStatus" )) || (!strcmp( (char*)xmlroot->name, "killJob" )) || (!strcmp( (char*)xmlroot->name, "fixJob" )) ) {
    xmlnode = xmlroot->xmlChildrenNode->next;
    while( xmlnode != NULL ) {
      jobData data;
      xmliterator = xmlnode->xmlChildrenNode->next;
      while( xmliterator != NULL ) {

	if( !strcmp( (char*)xmliterator->name,"infoExecuter") ) 
	  if( xmliterator->xmlChildrenNode != NULL ) data.infoExecuter = (char*)xmliterator->xmlChildrenNode->content;
	if( !strcmp( (char*)xmliterator->name,"jobExeID") ) 
	  if( xmliterator->xmlChildrenNode != NULL ) data.jobExeID = (char*)xmliterator->xmlChildrenNode->content;
	if( !strcmp( (char*)xmliterator->name,"jobDefID") ) 
	  if( xmliterator->xmlChildrenNode != NULL ) data.jobDefID = (char*)xmliterator->xmlChildrenNode->content;
	if( !strcmp( (char*)xmliterator->name,"jobName") ) 
	  if( xmliterator->xmlChildrenNode != NULL ) data.jobName = (char*)xmliterator->xmlChildrenNode->content;

	xmliterator = xmliterator->next->next;

      }
      vJobExeData.push_back( data );
      xmlnode = xmlnode->next->next;
    }

    for( unsigned int i=0; i<vJobExeData.size(); i++ ) {
      cout << "jobDefID    [" << i << "] : " << vJobExeData[i].jobDefID << endl;
      cout << "jobName     [" << i << "] : " << vJobExeData[i].jobName << endl;
      cout << "jobExeID    [" << i << "] : " << vJobExeData[i].jobExeID << endl;
      cout << "infoExecuter[" << i << "] : " << vJobExeData[i].infoExecuter << endl;
    }
  }

  if( !strcmp( (char*)xmlroot->name, "numJobsWanted" ) ) {
    cpuConsumption cpu;
    xmlnode = xmlroot->xmlChildrenNode->next;
    if( !strcmp( (char*)xmlnode->name, "minimumResources" ) ) {
      xmlnode = xmlnode->xmlChildrenNode->next;
      if( !strcmp( (char*)xmlnode->name, "cpuConsumption" ) ) {
	xmlnode = xmlnode->xmlChildrenNode->next;
	
	while( xmlnode != NULL) {
	  if( !strcmp( (char*)xmlnode->name,"count") ) 
	    if( xmlnode->xmlChildrenNode != NULL ) cpu.count = (char*)xmlnode->xmlChildrenNode->content;
	  if( !strcmp( (char*)xmlnode->name,"unit") ) 
	    if( xmlnode->xmlChildrenNode != NULL ) cpu.unit = (char*)xmlnode->xmlChildrenNode->content;
	  xmlnode = xmlnode->next->next;
	}
      }
    }

    // print what has been read
    cout << "count: " << cpu.count << endl;
    cout << "unit : " << cpu.unit << endl;
  }

  if( !strcmp( (char*)xmlroot->name, "executeJobs" ) ) {

    xmlroot = xmlroot->xmlChildrenNode->next;
    while( xmlroot != NULL ) {

      if( xmlroot->xmlChildrenNode->next != NULL ) {
	
	xmlnode = xmlroot->xmlChildrenNode->next;
	jobDef job;

	while( xmlnode != NULL ) {
	  if( !strcmp( (char*)xmlnode->name, "jobDefID" ) ) {
	    if( xmlnode->xmlChildrenNode != NULL ) job.jobDefID = (char*)xmlnode->xmlChildrenNode->content;
	  }
	  if( !strcmp( (char*)xmlnode->name, "jobName" ) ) {
	    if( xmlnode->xmlChildrenNode != NULL ) job.jobName = (char*)xmlnode->xmlChildrenNode->content;
	  }
	  if( !strcmp( (char*)xmlnode->name, "transImplementation" ) ) {
	    if( xmlnode->xmlChildrenNode != NULL ) job.transImpl = (char*)xmlnode->xmlChildrenNode->content;
	  }
	  if( !strcmp( (char*)xmlnode->name, "transUses" ) ) {
	    if( xmlnode->xmlChildrenNode != NULL ) job.transUses = (char*)xmlnode->xmlChildrenNode->content;
	  }
	  
	  if( !strcmp( (char*)xmlnode->name, "expectedResources" ) ) {
	    xmliterator = xmlnode->xmlChildrenNode->next;
	    while ( xmliterator != NULL ) {
	      if( !strcmp( (char*)xmliterator->name, "cpuConsumption" ) ) {
		xmliterator2 = xmliterator->xmlChildrenNode->next;
		while ( xmliterator2 != NULL ) {
		  if ( !strcmp( (char*)xmliterator2->name, "count" ) )
		    if( xmliterator2->xmlChildrenNode != NULL ) job.CPU.count = (char*)xmliterator2->xmlChildrenNode->content;
		  if ( !strcmp( (char*)xmliterator2->name, "unit" ) )
		    if( xmliterator2->xmlChildrenNode != NULL ) job.CPU.unit = (char*)xmliterator2->xmlChildrenNode->content;
		  xmliterator2 = xmliterator2->next->next;
		}
	      }
	      if( !strcmp( (char*)xmliterator->name, "diskConsumption" ) ) {
		xmliterator2 = xmliterator->xmlChildrenNode->next;
		while ( xmliterator2 != NULL ) {
		  if ( !strcmp( (char*)xmliterator2->name, "count" ) )
		    if( xmliterator2->xmlChildrenNode != NULL) job.disk.count = (char*)xmliterator2->xmlChildrenNode->content;
		  if ( !strcmp( (char*)xmliterator2->name, "unit" ) )
		    if( xmliterator2->xmlChildrenNode != NULL) job.disk.unit = (char*)xmliterator2->xmlChildrenNode->content;
		  xmliterator2 = xmliterator2->next->next;
		}
	      }
	      xmliterator = xmliterator->next->next;
	    }
	  }
	  
	  if( !strcmp( (char*)xmlnode->name, "jobPars" ) ) {
	    xmliterator = xmlnode->xmlChildrenNode->next;
	    while ( xmliterator != NULL ) {
	      if( !strcmp( (char*)xmliterator->name, "actualPar" ) ) {
		actualPar pars;
		xmliterator2 = xmliterator->xmlChildrenNode->next;
		while ( xmliterator2 != NULL ) {
		  if ( !strcmp( (char*)xmliterator2->name, "name" ) )
		    if( xmliterator2->xmlChildrenNode != NULL ) pars.name = (char*)xmliterator2->xmlChildrenNode->content;
		  if ( !strcmp( (char*)xmliterator2->name, "value" ) )
		    if( xmliterator2->xmlChildrenNode != NULL ) pars.value = (char*)xmliterator2->xmlChildrenNode->content;
		  xmliterator2 = xmliterator2->next->next;
		}
		job.jobPars.push_back( pars );
	      }
	      xmliterator = xmliterator->next->next;
	    }
	  }
	  
	  if( !strcmp( (char*)xmlnode->name, "jobInputs" ) || !strcmp( (char*)xmlnode->name, "jobOutputs" ) ) {
	    xmliterator = xmlnode->xmlChildrenNode->next;
	    while ( xmliterator != NULL ) {
	      if( !strcmp( (char*)xmliterator->name, "fileInfo" ) ) {
		fileInfo info;
		xmliterator2 = xmliterator->xmlChildrenNode->next;
		while ( xmliterator2 != NULL ) {
		  if ( !strcmp( (char*)xmliterator2->name, "LFN" ) )
		    if( xmliterator2->xmlChildrenNode != NULL ) info.LFN = (char*)xmliterator2->xmlChildrenNode->content;
		  if ( !strcmp( (char*)xmliterator2->name, "type" ) )
		    if( xmliterator2->xmlChildrenNode != NULL ) info.type = (char*)xmliterator2->xmlChildrenNode->content;
		  if ( !strcmp( (char*)xmliterator2->name, "logCol" ) )
		    if( xmliterator2->xmlChildrenNode != NULL ) info.logCol = (char*)xmliterator2->xmlChildrenNode->content;
		
		  if ( !strcmp( (char*)xmliterator2->name, "SEList" ) ) {
		    xmliterator3 = xmliterator2->xmlChildrenNode->next;
		    while( xmliterator3 != NULL ) {
		      if( !strcmp( (char*)xmliterator3->name, "SE" ) )
			info.SEList.push_back( (char*)xmliterator3->xmlChildrenNode->content );
		      xmliterator3 = xmliterator3->next->next;
		    }
		  }
		    
		  if ( !strcmp( (char*)xmliterator2->name, "turlList" ) ) {
		    xmliterator3 = xmliterator2->xmlChildrenNode->next;
		    while( xmliterator3 != NULL ) {
		      if( !strcmp( (char*)xmliterator3->name, "turl" ) )
			info.turlList.push_back( (char*)xmliterator3->xmlChildrenNode->content );
		      xmliterator3 = xmliterator3->next->next;
		    }
		  }
		
		  xmliterator2 = xmliterator2->next->next;
		}
		
		if( !strcmp( (char*)xmlnode->name,"jobInputs" ) ) job.jobInputs.push_back( info );
		if( !strcmp( (char*)xmlnode->name,"jobOutputs" ) ) job.jobOutputs.push_back( info );
	      }
	      xmliterator = xmliterator->next->next;
	    }
	  }
	  xmlnode = xmlnode->next->next;
	}
	
	vJobDef.push_back( job );
	xmlroot = xmlroot->next->next;
      }
    }

    // constructing xrsl
    for( unsigned int i=0; i<vJobDef.size(); i++) {

      jobDef job = vJobDef[i];
      string xrsl = "&";
      string arguments;

      if( job.transImpl.size()>0 )
	xrsl = xrsl + "(executable=\"" + job.transImpl + "\")";
      for( unsigned int j=0; j<job.jobPars.size(); j++ ) {
	if( !strcmp( job.jobPars[i].name.c_str(), "numEvt" ) ) {
	  arguments = arguments + job.jobPars[i].value + " ";
	}
      }
      for( unsigned int j=0; j<job.jobInputs.size(); j++ ) {
	if( !strcmp( job.jobInputs[j].type.c_str(), "signalLFN" ) ) {
	  arguments = arguments + job.jobInputs[j].LFN + " ";
	}
      }
      for( unsigned int j=0; j<job.jobOutputs.size(); j++ ) {
	if( !strcmp( job.jobOutputs[j].type.c_str(), "outputLFN" ) ) {
	  arguments = arguments + job.jobOutputs[j].LFN + " ";
	}
      }
      if( arguments.size()>0 ) {
	arguments.erase( arguments.size()-1 );
	xrsl = xrsl + "(arguments=" + arguments + ")";
      }
      xrsl = xrsl + "(stdout=\"stdout\")(join=yes)";
      if( job.transUses.size()>0 )
	xrsl = xrsl + "(runTimeEnvironment=\"" + job.transUses + "\")";
      if( (job.disk.count.size()>0) && (job.disk.unit.size()>0) )
	xrsl = xrsl + "(Disk=\"" + job.disk.count + " " + job.disk.unit + "\")";
      if( (job.CPU.count.size()>0) && (job.CPU.unit.size()>0) )
	xrsl = xrsl + "(CPUTime=\"" + job.CPU.count + " " + job.CPU.unit + "\")";
      if( (job.jobName.size()>0)||(job.jobDefID.size()>0) )
	xrsl = xrsl + "(jobName=\"" + job.jobName + "$" + job.jobDefID + "\")";
      if( job.jobInputs.size()>0 ) {
	xrsl = xrsl + "(inputFiles=";
	for( unsigned int j=0; j<job.jobInputs.size(); j++ ) {
	  if( job.jobInputs[j].turlList.size()>0 ) {
	    xrsl = xrsl + "(\"" + job.jobInputs[j].LFN + "\" \"" + job.jobInputs[j].turlList[0]+ "\")";
	  }
	}
	xrsl = xrsl + ")";
      }
      
      if( job.jobOutputs.size()>0 ) {
	xrsl = xrsl + "(outputFiles=";
	for( unsigned int j=0; j<job.jobOutputs.size(); j++ ) {
	  if( job.jobOutputs[j].SEList.size()>0 ) {
	    xrsl = xrsl + "(\"" + job.jobOutputs[j].LFN + "\" \"rc://";
	    for( unsigned int k=0; k<job.jobOutputs[j].SEList.size(); k++ ) {
	      xrsl = xrsl + job.jobOutputs[j].SEList[k];
	      if( k != job.jobOutputs[j].SEList.size()-1 ) xrsl = xrsl + "|";
	    }
	    xrsl = xrsl + "@grid.uio.no/" + job.jobOutputs[j].logCol + "/" + job.jobOutputs[j].LFN + "\")";
	  }
	}
	xrsl = xrsl + ")";
      }
      
      cout << "xrsl: " << xrsl << endl;
    }

  }

  xmlFreeDoc( xmldoc );
  /*if( argv[2] != NULL ) 
    xmlFreeDtd( xmldtd );*/

  xmlCleanupParser();

  return 0;

}

